require 'securerandom'
require 'active_support/notifications'
require 'active_support/log_subscriber'

module Twirl
  module Instrumentation
    class LogSubscriber < ::ActiveSupport::LogSubscriber
      def op(event)
        return unless logger.debug?

        op = event.payload[:op]
        bytes = event.payload[:bytes]
        queue_name = event.payload[:queue_name]

        return unless op

        description = "Twirl op(#{op})"
        details = ""

        if queue_name
          details += "queue_name=#{queue_name} "
        end

        if bytes
          details += "bytes=#{bytes} "
        end

        name = '%s (%.1fms)' % [description, event.duration]
        debug "  #{color(name, CYAN, true)}  [ #{details} ]"
      end
    end
  end

  Instrumentation::LogSubscriber.attach_to :twirl
end
