require "twirl/instrumenters/noop"

module Twirl
  class Item
    # Public: The key of the item.
    attr_reader :key

    # Public: The value of the item.
    attr_reader :value

    # Private: The client that popped the item.
    attr_reader :client

    def initialize(key, value, client, instrumenter = nil)
      @key = key
      @value = value
      @client = client
      @instrumenter = instrumenter || Instrumenters::Noop
    end

    # Public: Acknowledge that we are done processing the item.
    def close
      @instrumenter.instrument "op.twirl" do |payload|
        payload[:op] = :item_close
        payload[:queue_name] = @key

        @client.close @key
      end
    end

    # Public: Something went wrong processing.
    def abort
      @instrumenter.instrument "op.twirl" do |payload|
        payload[:op] = :item_abort
        payload[:queue_name] = @key

        @client.abort @key
      end
    end

    def eql?(other)
      self.class.eql?(other.class) &&
        @key == other.key &&
        @value == other.value &&
        @client == other.client
    end
    alias_method :==, :eql?
  end
end
